#!/usr/bin/python
## -*- utf-8 -*-

import logging
log = logging.getLogger(__name__) 
from util import *
import communication

##############################################################################
##

class Game(object):
  
  def __init__(self, gameMode):
    self.IDLE = Game.Idle(self)
    self.RECORD = Game.Record(self)
    self.WFTURN = Game.Wfturn(self)
    self.FINISH = Game.Finish(self)
    self.changeState(self.IDLE)
  
  def setComm(self,comm):
      self._comm = comm
      
  def setGui(self, gui):
      self._gui = gui


  def changeState(self,state):
    state_name = state.name()
    log.debug("new state %(state_name)s" % vars())
    self._state = state

  ############
  def intro(self,name): self._state.intro(name)
  def bye(self): self._state.bye()
  def start_game(self): self._state.start_game()
  def end_game(self): self._state.end_game()
  def sequence(self,seq): self._state.sequence(seq)
  def winner(self,name): self._state.winner(name)

  ########
  class State(communication.Interface):
    
    def __init__(self,ctx):
      self._ctx = ctx
      
    def name(self):
      return "unknown"
    
    def ding(self,slotnum):
      pass
      
    
  class Idle(State):
    def __init__(self,ctx):
      super(Game.Idle,self).__init__(ctx)      

    def name(self):
      return "idle"
    
    def start_game(self):
      self._comm.start_game()
      if self._ctx._comm.mode() == self._ctx._comm.ASHOST:
        changeState(self._ctx.WFTURN)
      else:
        changeState(self._ctx.RECORD)

  class Record(State):
    def __init__(self,ctx):
      super(Game.Record,self).__init__(ctx)      
    def name(self):
      return "record"

  class Wfturn(State):
    def __init__(self,ctx):
      super(Game.Wfturn,self).__init__(ctx)      
    def name(self):
      return "wfturn"
    
    def sequence(self):
      changeState(self._ctx.RECORD)

  class Finish(State):
    def __init__(self,ctx):
      super(Game.Finish,self).__init__(ctx)      
    def name(self):
      return "finish"


##############################################################################
##

if __name__ == '__main__':
  logging.basicConfig(level=logging.DEBUG)
  comm = communication.Comm()
  game = Game(comm)
  comm.set_ind(game)
